#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include <math.h>

#include "../../sdk_version.h"
#include "control_id.h"
#include "resource_icon.h"
#include "scopekey.h"

IMPLEMENT_DYNAMIC(CScopeKey, CWnd)

BEGIN_MESSAGE_MAP(CScopeKey, CWnd)
  ON_WM_CLOSE()
  ON_WM_CREATE()
  ON_WM_PAINT()
  ON_COMMAND_RANGE(IDCTL_TRACE_EN(0), IDCTL_TRACE_EN(255), OnTraceEnable)
  ON_COMMAND(IDCTL_TRACE_ALL, OnTraceAll)
  ON_COMMAND(IDCTL_TRACE_NONE, OnTraceNone)
END_MESSAGE_MAP()

const DWORD g_dwStyle = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU;
const DWORD g_dwExStyle = WS_EX_CONTROLPARENT | WS_EX_PALETTEWINDOW;

// -------------------------------------------------------------------------
// Start of IScopeKey interface
// -------------------------------------------------------------------------

void
CScopeKey::tracesChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent()->getParent();
  IScopeWnd* pScopeWnd = m_pParent->getParent()->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = pScopeWnd->getNumTrace();
  RECT r, wr;

  deleteButtons();
  if (n) {
    m_pCheckBoxes = new CButton[n];
    for (unsigned int i = 0; i < n; i++) {
      unsigned int sensorIndex;
      TCHAR* pSensorName;

      sensorIndex = pScopeWnd->getTrace(i);
      ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);
      pSensorName = devInfo.m_pSensors[sensorIndex].m_info.Description;
      r.left = 70;
      r.right = 230;
      r.top = m_traceStartY + 20 * i;
      r.bottom = m_traceStartY + 20 * (i + 1);
      m_pCheckBoxes[i].Create(pSensorName, WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_AUTOCHECKBOX, r, this, IDCTL_TRACE_EN(i));
      m_pCheckBoxes[i].SetCheck(BST_CHECKED);
      m_pCheckBoxes[i].SetFont(&m_font);
    }
    m_numTrace = n;
    m_allCtl.SetCheck(BST_CHECKED);
    r.left = 0;
    r.top = 0;
    r.right = 240;
    r.bottom = m_traceStartY + 10 + 20 * n;
    AdjustWindowRectEx(&r, g_dwStyle, FALSE, g_dwExStyle);
    GetWindowRect(&wr);
    MoveWindow(wr.left, wr.top, r.right - r.left, r.bottom - r.top, TRUE);
  }
  Invalidate(TRUE);
}

// -------------------------------------------------------------------------
// End of IScopeKey interface
// -------------------------------------------------------------------------

CScopeKey::CScopeKey(
  IGraphWndToolbar* pParent)
{
  const TCHAR* pWndClass = NULL;

  m_pParent = pParent;
  m_pCheckBoxes = NULL;
  m_numTrace = 0;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_blackBrush.CreateSolidBrush(RGB(0, 0, 0));
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  m_hIcon = ::LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SYSMON));

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, m_hIcon);
  CreateEx(
    g_dwExStyle,
    pWndClass,
    _T("Key"),
    g_dwStyle,
    0, 0,
    m_defaultWidth, m_defaultHeight,
    m_pParent->getParent()->getParent()->getNativeWindow(),
    NULL,
    NULL);
}

CScopeKey::~CScopeKey()
{
  deleteButtons();
}

void
CScopeKey::deleteButtons()
{
  if (m_numTrace) {
    ASSERT(NULL != m_pCheckBoxes);
    delete[] m_pCheckBoxes;
    m_pCheckBoxes = NULL;
    m_numTrace = 0;
  }
}

void
CScopeKey::OnClose()
{
  ShowWindow(SW_HIDE);
  m_pParent->keyClosed();
}

int
CScopeKey::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  RECT r;
  r.left = 10;
  r.right = 90;
  r.top = 10;
  r.bottom = 33;
  m_allCtl.Create(_T("Select All"), WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON, r, this, IDCTL_TRACE_ALL);
  m_allCtl.SetFont(&m_font);
  r.left = 110;
  r.right = 190;
  r.top = 10;
  r.bottom = 33;
  m_noneCtl.Create(_T("Select None"), WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON, r, this, IDCTL_TRACE_NONE);
  m_noneCtl.SetFont(&m_font);

  return 0;
}

void
CScopeKey::OnPaint()
{
  static const float equivalentTime = 30.0f / 12.0f;
  IScopeWnd* pScope = m_pParent->getParent()->getScope();
  const DeviceInfo& devInfo = m_pParent->getParent()->getParent()->getDeviceInfo();
  CPaintDC dc(this);
  RECT r;
  int x, y;

  r.left = 10;
  r.right = 60;
  r.top = m_traceStartY;
  r.bottom = m_traceStartY + 20 * m_numTrace;
  dc.FillRect(&r, &m_blackBrush);

  x = 20;
  y = m_traceStartY + 10;
  for (unsigned int i = 0; i < m_numTrace; i++) {
    unsigned int sensorIndex;

    sensorIndex = pScope->getTrace(i);
    ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);

    m_lineRenderer.setDC((HDC)dc);
    m_lineRenderer.setColors(2, devInfo.m_pSensors[sensorIndex].m_traceColor);
    m_lineRenderer.start((float)x, (float)y, 0.0f);
    m_lineRenderer.add((float)(x + 30), (float)y, equivalentTime);
    m_lineRenderer.finish();

    y += 20;
  }
}

void
CScopeKey::OnTraceEnable(
  UINT nId)
{
  unsigned int index;
  bool_t bEnable;

  if (nId < IDCTL_TRACE_EN(0) || nId > IDCTL_TRACE_EN(255)) {
    /* Unexpected control ID; do nothing */
    return;
  }

  index = nId - IDCTL_TRACE_EN(0);
  bEnable = (m_pCheckBoxes[index].GetCheck() == BST_CHECKED) ? TRUE : FALSE;
  m_pParent->getParent()->getScope()->enableTrace(index, bEnable);
}

void
CScopeKey::OnTraceAll()
{
  unsigned int i;

  for (i = 0; i < m_numTrace; i++) {
    m_pCheckBoxes[i].SetCheck(BST_CHECKED);
    m_pParent->getParent()->getScope()->enableTrace(i, TRUE);
  }
}

void
CScopeKey::OnTraceNone()
{
  unsigned int i;

  for (i = 0; i < m_numTrace; i++) {
    m_pCheckBoxes[i].SetCheck(BST_UNCHECKED);
    m_pParent->getParent()->getScope()->enableTrace(i, FALSE);
  }
}
