#pragma once

#include <objmodel.h>
#include "tracebuffer.h"
#include "stripedline.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

class CScopeWnd : public CWnd, public IScopeWnd
{
	DECLARE_DYNAMIC(CScopeWnd)

public:
	CScopeWnd(IGraphWnd* pParent);
	virtual ~CScopeWnd();
  // IScopeWnd interface
  void addTrace(unsigned int sensorIndex, int at = -1);
  void addTraceDone();
  void adjustScale(float value);
  void deviceChanged();
  void enableTrace(unsigned int traceIndex, bool_t bEnable);
  control_handle_t getNativeControl() { return *this; }
  unsigned int getNumTrace() { return m_numTrace; }
  IGraphWnd* getParent(void) const { return m_pParent; }
  unsigned int getTrace(unsigned int traceIndex) {
    ASSERT(traceIndex < m_numTrace);
    return m_pTraceInfo[traceIndex].m_pTraceBuffer->getSensorIndex();
  };
  void idle();
  void removeAllTraces();
  void update(timestamp_t timestamp);
  void updatePeriodChanged();

protected:
  // If TRUE, uses double-buffered drawing using in-memory device context
  static const bool_t m_bDoubleBuffered = TRUE;

  // If TRUE, calls Invalidate() when (app is idle) AND m_bNeedRedraw
  // This avoids WM_TIMER messages being delayed by frequent WM_PAINT messages
  static const bool_t m_bInvalidateOnlyWhenIdle = TRUE;

  // Size of POINT buffer for PolyLine drawing
  static const size_t m_pointBufferSize = 512;

  // Per-trace information
  struct CTraceInfo {
    CTraceBuffer* m_pTraceBuffer;
    bool_t m_bDisplayEnabled;
  };
  // Information about how multiple traces are scaled to this scope
  struct Scaling {
    int m_numYDivision;
    float m_yDivision;
    float m_yMin, m_yMax;
    float m_yScaling;
    bool_t m_bYValid;
  };
  // Layout of axes etc. on scope; all values in pixels relative to client rectangle
  struct Layout{
    int m_axisYTop;
    int m_axisXYBottom;
    int m_axisXYLeft;
    int m_axisXRight;
    unsigned int m_axisYHeight;
    unsigned int m_axisXWidth;
  };
  IGraphWnd* m_pParent;
  CBrush m_bgBrush;
  CFont m_font;
  Scaling m_scaling;
  Layout m_layout;
  CTraceInfo* m_pTraceInfo;
  unsigned int m_numTrace;
  CDC m_memDC;
  CBitmap m_bitmap;
  CBitmap* m_pOldBitmap;
  POINT* m_pPointBuffer;
  bool_t m_bMouseOver;
  POINT m_mousePos;
  TRACKMOUSEEVENT m_mouseTrack;
  bool_t m_bNeedRedraw;
  float m_minimum, m_maximum;
  CStripedLine m_lineRenderer;

protected:
  typedef const CTraceBuffer* CTraceBufferConstPtr;
  void calcLayout(Layout& layout, const RECT& clientRect);
  void calcYScaling(Scaling& scaling, const CTraceInfo* m_pTraceInfo, const Layout& layout, unsigned int numTrace);
  void drawAxes(CDC& dc);
  void drawAxisLabels(CDC& dc, const SensorInfo* pSensorInfo);
  void drawGuides(CDC& dc, const DeviceInfo& devInfo);
  void drawMessage(CDC& dc, const DeviceInfo& devInfo);
  void drawMinimalTicks(CDC& dc);
  void drawTrace(CDC& dc, SensorInfo* pSensorInfo, CTraceBuffer* pSensorData, timestamp_t t0);
  void needRedraw();
  bool xToTimestamp(int mouseX, timestamp_t t0, unsigned int updatePeriod, timestamp_t& timestamp);
  bool timestampToX(timestamp_t timestamp, timestamp_t t0, unsigned int updatePeriod, int& x);
  bool valueToY(float value, int& y);

  DECLARE_MESSAGE_MAP()

protected:
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnDestroy();
  afx_msg bool_t OnEraseBkgnd(CDC* pDC);
  afx_msg void OnMouseLeave();
  afx_msg void OnMouseMove(UINT nFlags, CPoint point);
  afx_msg void OnPaint();
  afx_msg void OnSize(UINT nType, int cx, int cy);
};
