#ifndef _ADATA_PLATFORM_TIME_H
#define _ADATA_PLATFORM_TIME_H

#include <vxWorks.h>
#include <sysLib.h>
#include <taskLib.h>
#include <tickLib.h>

typedef UINT64 admxrc3_time_t;

#if (_WRS_VXWORKS_MAJOR > 6)
# define timeGet() tick64Get()
#else
# define timeGet() tickGet()
#endif

static const admxrc3_time_t timeZero = 0U;

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
timeMake(
  long sec,
  long usec,
  bool_t bRoundUp)
{
  const unsigned int M = 1000000U;
  unsigned int rate = sysClkRateGet();
  admxrc3_time_t u;

  u = ((admxrc3_time_t)sec * M + usec) * rate;
  if (bRoundUp) {
    return (u + M - 1) / M;
  } else {
    return u / M;
  }
}

#define timeAfter(t1, t2) ((t1) > (t2))

#define timeBefore(t1, t2) ((t1) < (t2))

#define timeEqual(t1, t2) ((t1) == (t2))

#define timeAdd(t1, t2) ((t1) + (t2))

#define timeSubtract(t1, t2) ((t1) - (t2))

#if defined(_WRS_INLINE)
_WRS_INLINE double
#else
static __inline__ double
#endif
timeToDouble(
  admxrc3_time_t t)
{
  return (double)t / (double)sysClkRateGet();
}

#if defined(_WRS_INLINE)
_WRS_INLINE float
#else
static __inline__ float
#endif
timeToFloat(
  admxrc3_time_t t)
{
  return (float)t / (float)sysClkRateGet();
}

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
doubleToTime(
  double d)
{
  return (admxrc3_time_t)(d * (double)sysClkRateGet());
}

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
floatToTime(
  float f)
{
  return (admxrc3_time_t)(f * (float)sysClkRateGet());
}

#if defined(_WRS_INLINE)
_WRS_INLINE void
#else
static __inline__ void
#endif
sleepMs(
  unsigned int ms)
{
  const int k = 1000;
  INT64 ticks;

  ticks = ((INT64)ms * sysClkRateGet() + k - 1) / k;
  taskDelay((int)ticks); 
}

#if defined(ADMXRC3_IMPORT_DELAY_US)

# if defined(i386) || defined(__i386) || defined(__i386__) || defined(__I386__) || defined(__386) || defined(__x86_64) || defined(x86_64) || defined(__amd64__) || defined(__amd64)

#   include <arch/i86/pentiumLib.h>
#   include <logLib.h>

static int32_t admxrc3_tscTickRateUs; /* TSC ticks per microsecond */

#   if defined(_WRS_INLINE)
_WRS_INLINE void
#   else
static __inline__ void
#   endif
initDelayUs(
  void)
{
  long long int start, stop, elapsed;
  int priority;

  /* Do busy-wait callibration for delayUs */
  
  priority = taskPriorityGet(taskIdSelf(), &priority);
  taskPrioritySet(taskIdSelf(), 0);
  taskDelay(1);
  pentiumTscGet64(&start);
  taskDelay(1);
  pentiumTscGet64(&stop);
  taskPrioritySet(taskIdSelf(), priority);  
  elapsed = stop - start;
  assert(elapsed <= 0x7A1207FF0BDBEULL); /* Sanity check, 0xF4240FFF0BDBE == 1000001 * 0x7FFFFFFF - 1 */
  admxrc3_tscTickRateUs = (int32_t)(elapsed / 1000000);
  assert(0 != admxrc3_tscTickRateUs); /* Sanity check */
  logMsg("initDelayUs: CPU ticks per microsecond = %ld\n", (long)admxrc3_tscTickRateUs, 0, 0, 0, 0, 0);
}

/* Busy wait in VxWorks */
#   if defined(_WRS_INLINE)
_WRS_INLINE void
#   else
static __inline__ void
#   endif
delayUs(
  unsigned long us)
{
  long long int start, stop, now;

  pentiumTscGet64(&start);
  stop = start + (long long int)us * admxrc3_tscTickRateUs;
  do {
    pentiumTscGet64(&now);
  } while (now - stop < 0);
}

# elif defined (__ppc) || defined(__PPC__)

#   include <arch/ppc/vxPpcLib.h>
#   include <logLib.h>

static INT32 admxrc3_tbTickRateUs; /* Timebase ticks per microsecond */

#   if defined(_WRS_INLINE)
_WRS_INLINE void
#   else
static __inline__ void
#   endif
initDelayUs(
  void)
{
  UINT32 startU, startL;
  UINT32 stopU, stopL;
  UINT64 start, stop, elapsed;
  int priority;
  
  /* Do busy-wait callibration for delayUs */
  priority = taskPriorityGet(taskIdSelf(), &priority);
  taskPrioritySet(taskIdSelf(), 0);
  taskDelay(1);
  vxTimeBaseGet(&startU, &startL);
  taskDelay(1);
  vxTimeBaseGet(&stopU, &stopL);
  taskPrioritySet(taskIdSelf(), priority);
  start = (UINT64)startL | ((uint64_t)startU << 32);
  stop = (UINT64)stopL | ((uint64_t)stopU << 32);
  elapsed = stop - start;
  assert(elapsed <= 0x7A1207FF0BDBEULL); /* Sanity check, 0xF4240FFF0BDBE == 1000001 * 0x7FFFFFFF - 1 */
  admxrc3_tbTickRateUs = (INT32)(elapsed / 1000000);
  assert(0 != admxrc3_tbTickRateUs); /* Sanity check */
  logMsg("initDelayUs: timebase ticks per microsecond = %ld\n", (long)admxrc3_tbTickRateUs, 0, 0, 0, 0, 0);
}

/* Busy wait in VxWorks */
#   if defined(_WRS_INLINE)
_WRS_INLINE void
#   else
static __inline__ void
#   endif
delayUs(
  unsigned long us)
{
  UINT32 startU, startL;
  UINT32 nowU, nowL;
  UINT64 start, stop, now;

  vxTimeBaseGet(&startU, &startL);
  start = (uint64_t)startL | ((uint64_t)startU << 32);
  stop = start + (long long int)us * admxrc3_tbTickRateUs;
  do {
    vxTimeBaseGet(&nowU, &nowL);
    now = (uint64_t)nowL | ((uint64_t)nowU << 32);
  } while (now - stop < 0);
}

# else

#   warning "Don't know how to implement microsecond-resolution delay for this architecture."
#   warning "delayUs will do nothing!"

#   define initDelayUs() /* Do nothing */
#   define delayUs(us)   /* Do nothing */

# endif

#endif /* defined(ADMXRC3_WANT_DELAY_US) */

#endif
